char *dialv = "Dial Command, V0.1(009) 16 May 85";

/*  C K U D I A  --  Dialing program for connection to remote system */

/*
 This module should work under all versions of Unix.  It calls externally
 defined system-depended functions for i/o, but depends upon the existence
 of various modem control functions.

 Author: Herm Fischer (HFISCHER@USC-ECLB)
 Contributed to Columbia University for inclusion in C-Kermit.
 Copyright (C) 1985, Herman Fischer, 16400 Ventura Blvd #201, Encino CA 91436
 Permission is granted to any individual or institution to use, copy, or
 redistribute this software so long as it is not sold for profit, provided this
 copyright notice is retained. 
*/

#include "ckcdeb.h"
#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <setjmp.h>
#include "ckcker.h"
#include "ckucmd.h"

extern int local, speed, flow, mdmtyp;
extern char ttname[], sesfil[];

#define HAYES 1			/* for mdmtyp settings */
#define VENTEL 2
#define GDC 3			/* GeneralDataComm 212A/ED (low error rate) */
#define RACAL 4			/* Racal Vadic */
#define PENRIL 5		/* Penril */
#define CERMETEK 6		/* Cermetek Info-Mate 212A */
#define HAYESNV 9		/* internal use, non-verbal V0 setting */

struct keytab mdmtab[] = {		/* Modem types for command parsing */
    "cermetek",  CERMETEK, 0,
    "direct",	0, 0,
    "gendatacomm", GDC, 0,
    "hayes",	HAYES, 0,
    "penril",  PENRIL, 0,
    "racalvadic", RACAL, 0,
    "ventel",	VENTEL, 0
};
int nmdm = (sizeof(mdmtab) / sizeof(struct keytab));

	/* modem parameters, ordered by modem type number codes */
static int
dialtime[] = {			/* time modem allows for dialing number */
	00, 35, 20, 32, 40, 50, 20 };

static char *
pausechr[] = {			/* character to tell modem to pause */
	"", "," , "%" , "%" , "Kk", "?", "BbPpTt" };  
				/* penril pause unknown -HF */
static int
pausetim[] = {			/* time modem waits when pause char rec'd */
	0, 2, 5, 3, 5, 0 };

static char *
wakeup[] = {		/* string to wakeup modem & place in command mode */
	"", "AT\r", "\r\r\r", "\r\r", "\005\r", "\r\r", "  XY\016R\r" } ;

static int
wake_rate[] = {		/* millisecs between wakeup characters */
	0, 0, 300, 500, 50, 300, 200, 0, 0, 0 };

static char
wake_prompt[] = {	/* prompt after wakeup */
	0, 0, '$', '$', '*', '>', 0 };

static char *
dial_mode[] = {		/* some modems need to be put into dialing mode */
	"", "", "", "D\r", "D\r", "k\r", "", "", "", "" }; 

static char
dial_prompt[] = {		/* prompt to go ahead and dial */
	0, 0, 0, ':', '?', ':', 0, 0, 0, 0 };

static char *
dial_fmt[] = {		/* format for dialing string to modem */
	"",				/* nothing - direct */
	"AT DT %s\r",			/* hayes */
	"<K%s\r>",			/* ventel */
	"T%s\r",			/* general datacomm */
	"%s\r",				/* racal */
	"%s\r",				/* penril */
	"\016D '%s'\r",			/* cermetek */
	"", "",				/* spares */
	"AT DT %s\r"			/* hayesnv */ };

static int
dial_rate[] = {		/* milliseconds between dialing digits for modem */
	0, 0, 0, 0, 0, 0, 200, 0, 0, 0 };

#define DIALING 4		/* for ttvt parameter */
#define CONNECT 5

#define CONNECTED 1		/* for completion status */
#define FAILED	  2

static int tries = 0;

#define LBUFL 100
static char lbuf[LBUFL];

static jmp_buf sjbuf;

static
timerh() {			/* timer interrupt handler */
    longjmp(sjbuf,1);
}

static
ttolSlow(s,millisec) char *s; int millisec; {  /* output s-l-o-w-l-y */
    for (; *s; s++) {
	ttoc(*s);
	msleep(millisec);
	}
    }

static
waitFor(c) char c; {		/* wait for character prompt input */
    while( (ttinc(0) & 0177) != c );
}

static
didWeGet(s,r) char *s, *r; {	/* Looks in string s for response r */
    int lr = strlen(r);		/*  0 means not found, 1 means found it */
    int i;
    for (i = strlen(s)-lr; i >= 0; i--)
    	if ( s[i] == r[0] ) if ( !strncmp(s+i,r,lr) ) return( 1 );
    return( 0 );	    
}


/*  D I A L  --  Dial up the remote system */

dial(telnbr) char *telnbr; {

    char c;
    char *i, *j;
    int waitct, status;
    char errmsg[50], *erp;
    int augMdm;		/* mdmtyp with switch settings added */
    int mdmEcho = 0;	/* assume modem does not echo */
    int n;
    int (*savAlrm)();	/* save incomming alarm function */

	if (!mdmtyp) {
	    printf("Sorry, you must 'set modem' first\n");
	    return(-2);
	}
	augMdm = mdmtyp;	/* internal use, to add dialer switches info*/

	if (!local) {
	    printf("Sorry, you must 'set line' first\n");
	    return(-2);
	}
	if (speed < 0) {
	    printf("Sorry, you must 'set speed' first\n");
	    return(-2);
        }
	if (ttopen(ttname,&local,mdmtyp) < 0) {/* Open, no wait for carrier */
	    erp = errmsg;
	    sprintf(erp,"Sorry, can't open %s",ttname);
	    perror(errmsg);
	    return(-2);
    	}
/* cont'd... */

					/* interdigit waits for tone dial */
/* ...dial, cont'd */


	waitct = 1*strlen(telnbr) ;	/* compute time to dial worst case */
	waitct += dialtime[augMdm];	/* dialtone + completion wait times */
	for (i=telnbr; *i; i++) 	/* add in pause characters time */
	    for (j=pausechr[augMdm]; *j; j++)
		if (*i == *j) {
		    waitct += pausetim[augMdm]; 
		    break;
		    }

       printf("Dialing thru %s, speed %d, number %s.\r\n",ttname,speed,telnbr);
       printf("The timeout for completing the call is %d seconds.\r\n",waitct);

/* Condition console terminal and communication line */	    
				/* place line into "clocal" dialing state */
	if ( ttpkt(speed,DIALING) < 0 )  {
	    printf("Sorry, Can't condition communication line\n");
	    return(-2);
    	}

/* Hang up the modem (in case it wasn't "on hook") */

	if ( tthang() < 0 ) {
	    printf("Sorry, Can't hang up tty line\n");
	    return(-2);
	    }

/* Put modem into command state */

    savAlrm = signal(SIGALRM,timerh);
    alarm(10);			/* give modem 10 seconds to wake up */
    if (setjmp(sjbuf)) {
	alarm(0);
	signal(SIGALRM,savAlrm);	/* cancel timer */
	ttpkt(speed,CONNECT);	/* cancel dialing state ioctl */
	printf("Sorry, unable to complete dialed connection\r\n");
	return(-2);
	}
    ttflui();			/* flush input buffer if any */


#define OKAY 1			/* modem attention attempt status */
#define IGNORE 2
#define GOT_O -2
#define GOT_A -3

switch (augMdm) {
    case HAYES:
    case HAYESNV:
	while(tries++ < 4) {
	    ttol("AT\r",3);	/* signal for attention, look for response */
	    status = 0;
	    while ( status <= 0 ) {
		switch (ttinc(0) & 0177) {
		    case 'A':			/* echoing, ignore */
			status = GOT_A;
			break;
		    case 'T':
			if (status == GOT_A) {
			    mdmEcho = 1;	/* expect echoing later */
			    status = 0;
			    break;
			}
			status = IGNORE;
			break;
		    case '\n':
		    case '\r':
			status = 0;
			break;
		    case '0':			/* numeric result code */
			augMdm = HAYESNV;	/* nonverbal result codes */
			status = OKAY;
			break;
		    case 'O':			/* maybe English result code*/
			status = GOT_O;
			break;
		    case 'K':
			if (status == GOT_O) {
			    augMdm = HAYES;
			    status = OKAY;
			    break;
			}			/* else its default anyway */
		    default:
			status = IGNORE;
			break;
		    }
		}
	    if (status == OKAY) break;
	    if (status == IGNORE) ttflui();
	    sleep(1);		/* wait before retrying */
	}
        if (status != 0) break;
        printf("Sorry, can't initialize modem\n");
	ttpkt(speed,CONNECT);		/* cancel dialing state ioctl */
	alarm(0);
	signal(SIGALRM,savAlrm);	/* cancel timer */
        return(-2);

/* cont'd... */

					/* interdigit waits for tone dial */
/* ...dial, cont'd */

    default:			/* place modem into command mode */
	ttolSlow(wakeup[augMdm], wake_rate[augMdm]);
	if (wake_prompt[augMdm]) waitFor(wake_prompt[augMdm]);
	break;
    }
    alarm(0);			/* turn off alarm */
    msleep(500);		/* give things settling time */
    alarm(10);			/* alarm on dialing prompts */

		
/* Dial the number */

				/* put modem into dialing mode */
    ttolSlow(dial_mode[augMdm], dial_rate[augMdm]);	
    if (dial_prompt[augMdm]) {	/* wait for prompt, if any expected */
	waitFor(dial_prompt[augMdm]);
	msleep(300);
	}

    alarm(0);			/* turn off alarm on dialing prompts */
    alarm(waitct);		/* time to allow for connecting */
    ttflui();			/* clear out stuff from waking modem up */
    sprintf(lbuf, dial_fmt[augMdm], telnbr); /* form dialing string */
    ttolSlow(lbuf,dial_rate[augMdm]);	/* send dialing string */

    if (augMdm == RACAL) {	/* acknowledge printout of dialing string */
	sleep(3);
	ttflui();
	ttoc('\r');
	}

/* cont'd... */

					/* interdigit waits for tone dial */
/* ...dial, cont'd */


/* Check for connection */

    status = 0;
    while (status == 0) {
      switch (augMdm) {
	case HAYES:
	case GDC:
	case RACAL:
	case VENTEL:
	case PENRIL:
	    for (n = 0; n < LBUFL; n++) lbuf[n] = '\0';
					
	    for (n=0; n < LBUFL; n++) {		/* accumulate response */
		lbuf[n] = (ttinc(0) & 0177); 
		if ( lbuf[n] == '\r' || lbuf[n] == '\n' ) break;
		}
	    if (n > 2) {
		switch (augMdm) {
		  case HAYES:
		    if (didWeGet(lbuf,"CONNECT")) status = CONNECTED;
		    if (didWeGet(lbuf,"NO CARRIER")) status = FAILED;
		    break;
		  case GDC:
		    if (didWeGet(lbuf,"ON LINE")) status = CONNECTED;
		    if (didWeGet(lbuf,"NO CONNECT")) status = FAILED;
		    break;
		  case RACAL:
		    if (didWeGet(lbuf,"ON LINE")) status = CONNECTED;
		    if (didWeGet(lbuf,"FAILED CALL")) status = FAILED;
		    break;
		  case VENTEL:
		    if (didWeGet(lbuf,"ONLINE!")) status = CONNECTED;
		    if (didWeGet(lbuf,"BUSY")) status = FAILED;
		    if (didWeGet(lbuf,"DEAD PHONE")) status = FAILED;
		    break;
		  case PENRIL:
		    if (didWeGet(lbuf,"OK")) status = CONNECTED;
		    if (didWeGet(lbuf,"BUSY")) status = FAILED;
		    if (didWeGet(lbuf,"NO RING")) status = FAILED;
		    break;
		  case CERMETEK:
		    if (didWeGet(lbuf,"\016A")) {	
			status = CONNECTED;
			ttolSlow("\016U 1\r",200);	/* make transparent*/
			}
		}
	    }
	    break;

	case HAYESNV:
	    c = ttinc(0) & 0177;
	    if (mdmEcho) {		/* sponge up dialing string */
		mdmEcho = c!='\r';	/* until return is echoed */
		break;
		}
	    if (c == '1') status = CONNECTED;
	    if (c == '3') status = FAILED;
	    if (c == '5') status = CONNECTED;
	    break;

	}
    }
	
/* Place line into modem-control (non-clocal) state */

    if (status == 0) printf("Sorry, Can't get response from modem\r\n");
    else if (status == CONNECTED) printf("Connected!\r\n");
    else if (status == FAILED) printf("Sorry, No Carrier\r\n");
    else printf("Failed to complete call\r\n");

    ttpkt(speed,CONNECT);	/* cancel dialing state ioctl */
    alarm(0);
    signal(SIGALRM,savAlrm);	/* cancel timer */
    return((status==CONNECTED) ? 0 : -2);
}
